import base64
from calendar import monthrange
from datetime import date
from rest_framework import serializers
from .models import Account, Like, Payment, RequestChat
from django.core.files.base import ContentFile

class AccountSerializer(serializers.ModelSerializer):
    is_chat_requested = serializers.SerializerMethodField()
    is_liked = serializers.SerializerMethodField()
    number_of_likes = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()

    class Meta:
        model = Account
        fields = ['id', 'fullName', 'dob', 'profession', 'interests', 'wants', 'gender', 'bio', 'email', 'phoneNumber', 'termsAccepted', 'photos', 'created_at', 'is_chat_requested', 'is_liked', 'number_of_likes', 'age']

    def get_is_chat_requested(self, obj):
        request_user_id = self.context.get('request_user_id')
        if request_user_id:
            return RequestChat.objects.filter(sender_id=request_user_id, receiver=obj.id).exists()
        return False

    def get_is_liked(self, obj):
        request_user_id = self.context.get('request_user_id')
        if request_user_id:
            return Like.objects.filter(sender_id=request_user_id, receiver=obj.id).exists()
        return False

    def get_number_of_likes(self, obj):
        return Like.objects.filter(receiver=obj.id).count()
    
    def get_age(self, obj): 
        if obj.dob:
            today = date.today() 
            age = today.year - obj.dob.year - ((today.month, today.day) < (obj.dob.month, obj.dob.day))
            return age
        return None

class LikeSerializer(serializers.ModelSerializer):
    receiver_name = serializers.SerializerMethodField()
    receiver_avatar = serializers.SerializerMethodField()
    receiver_phoneNumber = serializers.SerializerMethodField()
    is_chat_requested = serializers.SerializerMethodField()
    is_chat_accepted = serializers.SerializerMethodField()
    is_liked = serializers.SerializerMethodField()
    number_of_likes = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()

    class Meta:
        model = Like
        fields = '__all__'

    def get_receiver_name(self, obj):
        return obj.receiver.fullName
    
    def get_receiver_phoneNumber(self, obj):
        return obj.receiver.phoneNumber

    def get_age(self, obj): 
        if obj.receiver.dob:
            today = date.today() 
            age = today.year - obj.receiver.dob.year - ((today.month, today.day) < (obj.receiver.dob.month, obj.receiver.dob.day))
            return age
        return None

    def get_receiver_avatar(self, obj):
        if obj.receiver.photos:
            return obj.receiver.photos.url
        return None
    
    def get_is_chat_requested(self, obj):
        return RequestChat.objects.filter(sender=obj.receiver, receiver=obj.sender, is_accepted=False).exists()
    
    def get_is_chat_accepted(self, obj):
        return RequestChat.objects.filter(sender=obj.receiver, receiver=obj.sender, is_accepted=True).exists()

    def get_is_liked(self, obj):
        return Like.objects.filter(sender=obj.receiver, receiver=obj.sender).exists()

    def get_number_of_likes(self, obj):
        return Like.objects.filter(receiver=obj.receiver).count()

from rest_framework import serializers
from .models import Payment
from datetime import timedelta, datetime
import random
import string

from rest_framework import serializers
from .models import Payment, Account
from datetime import timedelta, datetime
import random
import string
from calendar import monthrange

from rest_framework import serializers
from .models import Payment, Account
from datetime import datetime, timedelta

from datetime import datetime, timedelta
from calendar import monthrange
import random
import string
from rest_framework import serializers 

from django.utils import timezone
 

class PaymentSerializer(serializers.ModelSerializer):
    user_id = serializers.IntegerField(write_only=True)

    class Meta:
        model = Payment
        fields = ['user_id', 'checkout_request_id', 'transaction_id', 'plan', 'amount', 'is_active', 'order_id', 'plan_expiry_time', 'created_at']
        read_only_fields = ['order_id', 'plan_expiry_time', 'is_active', 'created_at']

    def create(self, validated_data):
        try:
            user_id = validated_data.pop('user_id')
            user = Account.objects.get(id=user_id)
        except Account.DoesNotExist:
            raise serializers.ValidationError("Account not found.")

        # Calculate plan_expiry_time based on the plan type
        plan_type = validated_data['plan'].lower()
        if plan_type == 'daily':
            plan_expiry_time = timezone.now() + timedelta(days=1)
        elif plan_type == 'weekly':
            plan_expiry_time = timezone.now() + timedelta(weeks=1)
        elif plan_type == 'monthly':
            _, days_in_month = monthrange(timezone.now().year, timezone.now().month)
            plan_expiry_time = timezone.now() + timedelta(days=days_in_month)
        else:
            plan_expiry_time = timezone.now()

        # Generate order_id (6 random uppercase characters)
        order_id = ''.join(random.choices(string.ascii_uppercase, k=6))

        # Create the Payment instance with the fetched user and other validated data
        payment = Payment.objects.create(
            user=user,
            plan=validated_data['plan'],
            amount=validated_data['amount'],
            plan_expiry_time=plan_expiry_time,
            order_id=order_id,
            is_active=False if 'transaction_id' not in validated_data or not validated_data['transaction_id'] else True,
            status = "incomplete" if 'transaction_id' not in validated_data or not validated_data['transaction_id'] else "complete",
            checkout_request_id=validated_data.get('checkout_request_id'),
            transaction_id=validated_data.get('transaction_id', ''),
        )

        return payment

    def update(self, instance, validated_data):
        # Update plan_expiry_time and is_active based on current time
        if instance.plan_expiry_time < timezone.now():
            instance.is_active = False  # Payment expired
        else:
            instance.is_active = True  # Payment still active

        instance.save()
        return instance




class RequestChatSerializer(serializers.ModelSerializer):
    receiver_name = serializers.SerializerMethodField()
    receiver_avatar = serializers.SerializerMethodField()
    is_chat_requested = serializers.SerializerMethodField()
    is_liked = serializers.SerializerMethodField()
    number_of_likes = serializers.SerializerMethodField()
    receiver_phoneNumber = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()

    class Meta:
        model = RequestChat
        fields = ['id', 'sender', 'receiver', 'is_accepted', 'created_at',
                  'receiver_name', 'receiver_avatar', 'is_chat_requested', 'is_liked', 'number_of_likes', 'receiver_phoneNumber', 'age']

    def get_receiver_name(self, obj):
        return obj.receiver.fullName
    
    def get_receiver_phoneNumber(self, obj):
        return obj.receiver.phoneNumber

    def get_receiver_avatar(self, obj):
        if obj.receiver.photos:
            return obj.receiver.photos.url
        return None

    def get_is_chat_requested(self, obj):
        return RequestChat.objects.filter(sender=obj.receiver, receiver=obj.sender, is_accepted=False).exists()

    def get_is_liked(self, obj):
        return Like.objects.filter(sender=obj.receiver, receiver=obj.sender).exists()

    def get_number_of_likes(self, obj):
        return Like.objects.filter(receiver=obj.receiver).count()

    def get_age(self, obj): 
        if obj.receiver.dob:
            today = date.today() 
            age = today.year - obj.receiver.dob.year - ((today.month, today.day) < (obj.receiver.dob.month, obj.receiver.dob.day))
            return age
        return None
